# Copyright (c) Microsoft Corporation. All rights reserved.
# Highly Confidential Material
"""This module uses the dns server provided by the User to resolve the FQDN."""

import requests
from pmec_url_checker.dns_resolver import DnsResolver
from pmec_url_checker.url_db import URLDB
import logging
from typing import List, Optional
from urllib.parse import urlparse


class UrlChecker:
    """UrlChecker class to validate list of URLs are accessible or not."""

    def __init__(
        self,
        cloud: str,
        region: Optional[str],  # pylint: disable=E1136
        dnsserver: list,
        source_ip: Optional[str],  # pylint: disable=E1136
        custom_url: Optional[List[str]],  # pylint: disable=E1136
        skiptest: Optional[str],  # pylint: disable=E1136
    ) -> None:
        self.source_ip = source_ip
        self.cloud = cloud
        self.region = region
        self.dns_resolver = DnsResolver(dnsserver, source_ip)
        self._url_db = URLDB()
        self._url_db.load(cloud, region, custom_url, skiptest)

    def run(self) -> None:
        """Check the list of URLs are accessible or not."""
        # Retrieve the List from URL DB
        data = self._url_db.retrieve()
        logging.info("----------------------------------------------------")
        logging.info("Total URLs to be checked: %s", len(data))
        logging.info("----------------------------------------------------")
        for _, row in data.iterrows():
            logging.info("Checking the URL accessibility: %s", row["URL"])

            result, reason = self._check_url(row["URL"])
            if result:
                row["Status"] = "Pass"
            else:
                row["Status"] = "Fail"
                row["Reason"] = reason

            logging.info("----------------------------------------------------")

    def _check_url(self, url: str) -> tuple[bool, Optional[str]]:
        """Check the URL is accessible or not."""
        parsed_url = urlparse(url)
        hostname = parsed_url.hostname

        result, answers = self.dns_resolver.resolve(hostname)
        if result:
            for resolved_ip in answers:
                if parsed_url.port is not None:
                    resolved_url = parsed_url._replace(
                        netloc=resolved_ip.address + ":" + str(parsed_url.port)
                    )
                else:
                    resolved_url = parsed_url._replace(netloc=resolved_ip.address)

                if self._trigger_http_request(resolved_url.geturl()):
                    logging.info("PASS: URL is accessible: %s", resolved_url.geturl())
                    return True, None
            return False, "Unsuccessful web request"
        else:
            logging.error(
                "Failed to resolve the FQDN: %s with error: %s", hostname, answers
            )
            return False, answers

    def _trigger_http_request(self, url: str) -> bool:
        """Trigger HTTP request to URL provided."""
        try:
            logging.debug("Testing Webrequest to: %s  Source: %s", url, self.source_ip)

            webrequest = self._create_session()  # Create TCP session
            r = webrequest.get(url, allow_redirects=False, timeout=10, verify=False)
            logging.debug("HTTP Status Code: %d", r.status_code)

            # any code between 100-599 is considered a success since it indicates that the
            # request is passed by firewall except the Timeout exception.
            if r.status_code not in range(100, 599):
                msg = (
                    "FAIL: Webrequest using Source IP {} received invalid status code for "
                    "{} and it is not within the acceptable range of 100-599. StatusCode={}".format(
                        self.source_ip, url, r.status_code
                    )
                )
                logging.debug(msg)
            else:
                msg = "PASS: Successfull webrequest using Source IP {} to: {}".format(
                    self.source_ip, url
                )
                logging.debug(msg)
                return True
        except requests.exceptions.RequestException as e:
            msg = "FAIL: Unsuccessfull webrequest using Source IP {} to: {}: Exception: {}".format(
                self.source_ip, url, str(e)
            )
            logging.debug(msg)
        return False

    def _create_session(self) -> requests.Session:
        """
        Create `Session` which will bind to the specified local address
        rather than auto-selecting it.
        # usage example:
        s = _create_session('100.64.246.10')
        s.get('https://login.microsoftonline.com')
        """
        session = requests.Session()
        for prefix in ("http://", "https://"):
            session.get_adapter(prefix).init_poolmanager(
                # those are default values from HTTPAdapter's constructor
                connections=requests.adapters.DEFAULT_POOLSIZE,
                maxsize=requests.adapters.DEFAULT_POOLSIZE,
                # This should be a tuple of (address, port). Port 0 means auto-selection.
                source_address=(self.source_ip if self.source_ip else "0.0.0.0", 0),
            )
        return session

    def dump_result(self, output: str) -> None:
        """Dump the result into file."""
        data = self._url_db.retrieve()
        data.to_csv(output)

    def get_urls_status(self) -> list[(str, str)]:
        """Get the status of all the URLs."""
        return self._url_db.get_urls_status()

    def get_time_server(self, time_server: str) -> Optional[List[str]]:
        """Get the IP address of Time Server."""
        result, answers = self.dns_resolver.resolve(time_server)
        if result:
            return [resolved_ip.address for resolved_ip in answers]
        else:
            return None



# Copyright (c) Microsoft Corporation. All rights reserved.
# Highly Confidential Material
"""This module uses the dns server provided by the User to resolve the FQDN."""
import logging
from typing import Optional, Union

import dns.exception
import dns.resolver


class DnsResolver:
    """DnsResolver class to resolve the FQDN using the provided DNS server."""

    def __init__(
        self, dnsserver: list, source_ip: Optional[str]  # pylint: disable=E1136
    ) -> None:
        self.dnsserver = dnsserver
        self.resolver = dns.resolver.Resolver()
        self.resolver.nameservers = self.dnsserver
        self.source_ip = source_ip

    def resolve(
        self, hostname: str
    ) -> tuple[bool, Union[list, str]]:  # pylint: disable=E1136
        logging.info(
            "Resolving the FQDN (%s) using the provided DNS server\n\n", hostname
        )
        failure_reason = ""
        try:
            answers = self.resolver.query(hostname, "A", source=self.source_ip)
            logging.debug("Received DNS Response:")
            logging.debug("Server:\t %s", answers.nameserver)
            logging.debug("Address:\t %s#%s\n", answers.nameserver, answers.port)
            logging.debug("Source Ip:")
            logging.debug("Answers:")
            for data in answers:
                logging.debug("\tIP:  %s\tType:  %s", data.address, data.rdtype)

            if len(answers) != 0:
                return (True, answers)
        except dns.resolver.NoAnswer as e:
            logging.error("No Answer received for the Query: %s", e)
            failure_reason = "No Answer received"
        except dns.resolver.NXDOMAIN as e:
            logging.error("The Domain name does not exist: %s", e)
            failure_reason = "Domain name does not exist"
        except dns.resolver.YXDOMAIN as e:
            logging.error("The Domain name is too long: %s", e)
            failure_reason = "Domain name is too long"
        except dns.resolver.NoNameservers as e:
            logging.error("No Name Servers are found: %s", e)
            failure_reason = "No Name Servers are found"
        except dns.resolver.LifetimeTimeout as e:
            logging.error("Timeout occured while resolving the Domain name: %s", e)
            failure_reason = "Timeout"
        except dns.exception.DNSException as e:
            logging.error("Exception occured while resolving the FQDN: %s", e)
            failure_reason = "Unexpected Error"
        return (False, failure_reason)



# Copyright (c) Microsoft Corporation. All rights reserved.
# Highly Confidential Material
"""This module stores the list of URLs needed to be validated."""


from typing import List, Optional

import pandas as pd


RegionAbbreviation = {
    "australiacentral": "acl",
    "australiacentral2": "acl2",
    "australiaeast": "ae",
    "australiasoutheast": "ase",
    "brazilsouth": "brs",
    "brazilsoutheast": "bse",
    "centraluseuap": "ccy",
    "canadacentral": "cnc",
    "canadaeast": "cne",
    "centralus": "cus",
    "eastasia": "ea",
    "eastus2euap": "ecy",
    "eastus": "eus",
    "eastus2": "eus2",
    "francecentral": "frc",
    "francesouth": "frs",
    "germanynorth": "gn",
    "germanywestcentral": "gwc",
    "centralindia": "inc",
    "southindia": "ins",
    "westindia": "inw",
    "italynorth": "itn",
    "japaneast": "jpe",
    "japanwest": "jpw",
    "jioindiacentral": "jic",
    "jioindiawest": "jiw",
    "koreacentral": "krc",
    "koreasouth": "krs",
    "northcentralus": "ncus",
    "northeurope": "ne",
    "norwayeast": "nwe",
    "norwaywest": "nww",
    "qatarcentral": "qac",
    "southafricanorth": "san",
    "southafricawest": "saw",
    "southcentralus": "scus",
    "swedencentral": "sdc",
    "swedensouth": "sds",
    "southeastasia": "sea",
    "switzerlandnorth": "szn",
    "switzerlandwest": "szw",
    "uaecentral": "uac",
    "uaenorth": "uan",
    "uksouth": "uks",
    "ukwest": "ukw",
    "westcentralus": "wcus",
    "westeurope": "we",
    "westus": "wus",
    "westus2": "wus2",
    "westus3": "wus3",
    "usdodcentral": "udc",
    "usdodeast": "ude",
    "usgovarizona": "uga",
    "usgoviowa": "ugi",
    "usgovtexas": "ugt",
    "usgovvirginia": "ugv",
    "usnateast": "exe",
    "usnatwest": "exw",
    "usseceast": "rxe",
    "ussecwest": "rxw",
    "chinanorth": "bjb",
    "chinanorth2": "bjb2",
    "chinanorth3": "bjb3",
    "chinaeast": "sha",
    "chinaeast2": "sha2",
    "chinaeast3": "sha3",
    "germanycentral": "gec",
    "germanynortheast": "gne",
}


class URLDB:
    """URL DB class to store the list of URLs needed to be validated."""

    def __init__(self) -> None:
        self.cloud = ""
        self.region = ""
        self.__data = pd.DataFrame(
            columns=[
                "Cloud Environment",
                "Layer",
                "URL",
                "Service",
                "Status",
                "Reason",
            ]
        )

    def load(
        self,
        cloud: str,
        region: Optional[str],  # pylint: disable=E1136
        custom_url: Optional[List[str]],  # pylint: disable=E1136
        skiptest: Optional[str],  # pylint: disable=E1136
    ) -> None:
        """Load the list of URLs from the file."""
        self.cloud = cloud
        self.region = region.replace(" ", "").lower() if region else ""

        # PMEc URLs
        if skiptest != "PMEC":
            self.load_pmec_specific_urls()

        # ARC URL
        if skiptest != "ARC":
            self.load_arc_specific_urls()

        # NFM URLs
        if skiptest != "NFM":
            self.load_nfm_specific_urls()

        # ASE URLs
        if skiptest != "ASE":
            self.load_ase_specific_urls()

        if custom_url and len(custom_url) > 0:
            self.load_custom_urls(custom_url)

    def load_pmec_specific_urls(self) -> None:
        """Load the AP5GC specific URLs."""

        # URLs for Docker Registery
        self.__data = pd.concat(
            [
                self.__data,
                pd.DataFrame(
                    {
                        "Cloud Environment": self.cloud,
                        "Layer": "PMEC",
                        "URL": [
                            "https://privatemecprivatemecartifactstore4ab6cc42d9.azurecr.io/v2/_catalog",
                            "https://privatemecdevpublisherprivatemecdevartifactstore1c.azurecr.io/v2/_catalog",
                            "https://privatemecdevpublishereastus2euapprivatea803a14a92.azurecr.io/v2/_catalog",
                            "https://privatemecdevpublishercentraluseuappriva6b3529cad4.azurecr.io/v2/_catalog",
                            "https://privatemeceastus2euapprivatemecartifactsf14e732aab.azurecr.io/v2/_catalog",
                            "https://privatemeccentraluseuapprivatemecartifac166168b140.azurecr.io/v2/_catalog",
                            "https://pcmeprodeastusacr.azurecr.io/v2/_catalog",  # For PCME pod
                        ],
                        "Service": "Azure Container Registry",
                        "Status": "Not Tested",
                        "Reason": "NA",
                    },
                ),
            ],
            ignore_index=True,
        )

        # URLs for  Geneva Monitoring and Telemetry service
        self.__data = pd.concat(
            [
                self.__data,
                pd.DataFrame(
                    {
                        "Cloud Environment": self.cloud,
                        "Layer": "PMEC",
                        "URL": [
                            "https://global.prod.microsoftmetrics.com/",
                            "https://global.int.microsoftmetrics.com/",
                            "https://global.ppe.microsoftmetrics.com/",
                            "https://prod.hot.ingestion.msftcloudes.com/",
                        ],
                        "Service": "Monitoring and telemetry",
                        "Status": "Not Tested",
                        "Reason": "NA",
                    }
                ),
            ],
            ignore_index=True,
        )

    def load_ase_specific_urls(self) -> None:
        """Load the ASE specific URLs."""

        if self.cloud.lower() == "azurecloud":
            # URLs for service Authentication Service - Microsoft Entra ID
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://login.microsoftonline.com",
                            ],
                            "Service": "Authentication Service - Microsoft Entra ID",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://pod01-edg1.eus.databoxedge.azure.com/",
                                "https://pod01-edg1.wus2.databoxedge.azure.com/",
                                "https://pod01-edg1.sea.databoxedge.azure.com/",
                                "https://pod01-edg1.we.databoxedge.azure.com/",
                            ],
                            "Service": "Azure Stack Edge Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://euspod01edg1sbcnpu53n.servicebus.windows.net/",
                                "https://wus2pod01edg1sbcnqh26z.servicebus.windows.net/",
                                "https://seapod01edg1sbcnkw22o.servicebus.windows.net/",
                                "https://wepod01edg1sbcnhk23j.servicebus.windows.net/",
                                "https://azgnrelay-eastus-l1.servicebus.windows.net",
                            ],
                            "Service": "Azure Service Bus",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://crl.microsoft.com/pki/",  # DevSkim: ignore DS137138
                                "http://www.microsoft.com/pki/",  # DevSkim: ignore DS137138
                            ],
                            "Service": "Certificate Revocation",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://seapod1edg1monsa01kw22o.table.core.windows.net",
                                "https://seapod1edg1monsa02kw22o.table.core.windows.net",
                                "https://seapod1edg1monsa03kw22o.table.core.windows.net",
                                "https://euspod01edg1monsa01pu53n.table.core.windows.net",
                                "https://euspod01edg1monsa02pu53n.table.core.windows.net",
                                "https://euspod01edg1monsa03pu53n.table.core.windows.net",
                                "https://wus2pod1edg1monsa01qh26z.table.core.windows.net",
                                "https://wus2pod1edg1monsa02qh26z.table.core.windows.net",
                                "https://wus2pod1edg1monsa03qh26z.table.core.windows.net",
                                "https://wepod01edg1monsa01hk23j.table.core.windows.net",
                                "https://wepod01edg1monsa02hk23j.table.core.windows.net",
                                "https://wepod01edg1monsa03hk23j.table.core.windows.net",
                                # "http://*.msftncsi.com", # Use absolute URL here
                            ],
                            "Service": "Azure Storage Accounts",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://management.azure.com/",
                                "https://management.core.windows.net",
                            ],
                            "Service": "Azure Management Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://azureprofilerfrontdoor.cloudapp.net",
                                "https://azstrpprod.trafficmanager.net/",
                            ],
                            "Service": "Azure Traffic Manager",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://mcr.microsoft.com/v2/_catalog",  # DevSkim: ignore DS137138
                            ],
                            "Service": "Azure Container Registry",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://browser.events.data.microsoft.com/OneCollector/1.0/",
                            ],
                            "Service": "Azure Telemetry Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://windowsupdate.microsoft.com",  # DevSkim: ignore DS137138
                                "http://update.microsoft.com",  # DevSkim: ignore DS137138
                                "https://update.microsoft.com",
                                "http://download.microsoft.com",  # DevSkim: ignore DS137138
                                "https://download.microsoft.com",
                                "http://download.windowsupdate.com",  # DevSkim: ignore DS137138
                                "http://wustat.windows.com",  # DevSkim: ignore DS137138
                                "http://ntservicepack.microsoft.com",  # DevSkim: ignore DS137138
                                # "http://*.windowsupdate.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.windowsupdate.microsoft.com",  # Use absolute URL here
                                # "http://*.update.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.update.microsoft.com",  # Use absolute URL here
                                # "http://*.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.download.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.ws.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.ws.microsoft.com",  # Use absolute URL here
                                # "http://*.mp.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                            ],
                            "Service": "Microsoft Update Server",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                ],
                ignore_index=True,
            )

        elif self.cloud.lower() == "azureusgovernmentcloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://management.usgovcloudapi.net",
                            ],
                            "Service": "Azure Management Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://crl.microsoft.com/pki/",  # DevSkim: ignore DS137138
                                "http://www.microsoft.com/pki/",  # DevSkim: ignore DS137138
                            ],
                            "Service": "Certificate Revocation",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://login.microsoftonline.us",
                            ],
                            "Service": "Authentication Service - Microsoft Entra ID",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://pod01-edg1.ugv.databoxedge.azure.us/",
                            ],
                            "Service": "Azure Stack Edge Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://ugvpod1edg1monsa01hx7pv.table.core.usgovcloudapi.net",
                                "https://ugvpod1edg1monsa02hx7pv.table.core.usgovcloudapi.net",
                                "https://ugvpod1edg1monsa03hx7pv.table.core.usgovcloudapi.net",
                            ],
                            "Service": "Azure Storage Accounts",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://ugvpod01edg1sbcnhx7pv.servicebus.usgovcloudapi.net/",
                                "https://azgns-usgovvirginia-fairfax-1p-public.servicebus.usgovcloudapi.net",
                            ],
                            "Service": "Azure Service Bus",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://crl.microsoft.com/pki/",  # DevSkim: ignore DS137138
                                "http://www.microsoft.com/pki/",  # DevSkim: ignore DS137138
                            ],
                            "Service": "Certificate Revocation",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://windowsupdate.microsoft.com",  # DevSkim: ignore DS137138
                                "http://update.microsoft.com",  # DevSkim: ignore DS137138
                                "https://update.microsoft.com",
                                "http://download.microsoft.com",  # DevSkim: ignore DS137138
                                "https://download.microsoft.com",
                                "http://download.windowsupdate.com",  # DevSkim: ignore DS137138
                                "http://wustat.windows.com",  # DevSkim: ignore DS137138
                                "http://ntservicepack.microsoft.com",  # DevSkim: ignore DS137138
                                # "http://*.windowsupdate.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.windowsupdate.microsoft.com",  # Use absolute URL here
                                # "http://*.update.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.update.microsoft.com",  # Use absolute URL here
                                # "http://*.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.download.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.ws.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.ws.microsoft.com",  # Use absolute URL here
                                # "http://*.mp.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                            ],
                            "Service": "Microsoft Update Server",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://browser.events.data.microsoft.com/OneCollector/1.0/",
                            ],
                            "Service": "Azure Telemetry Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://mcr.microsoft.com/v2/_catalog",  # DevSkim: ignore DS137138
                            ],
                            "Service": "Azure Container Registry",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://azstrpffprod.usgovtrafficmanager.net/",
                            ],
                            "Service": "Remote Management",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://browser.events.data.microsoft.com/OneCollector/1.0/",
                            ],
                            "Service": "Azure Telemetry Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                ],
                ignore_index=True,
            )

        elif self.cloud.lower() == "azurechinacloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://management.chinacloudapi.cn",
                            ],
                            "Service": "Azure Management Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://login.chinacloudapi.cn",
                            ],
                            "Service": "Authentication Service - Microsoft Entra ID",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://windowsupdate.microsoft.com",  # DevSkim: ignore DS137138
                                "http://update.microsoft.com",  # DevSkim: ignore DS137138
                                "https://update.microsoft.com",
                                "http://download.microsoft.com",  # DevSkim: ignore DS137138
                                "https://download.microsoft.com",
                                "http://download.windowsupdate.com",  # DevSkim: ignore DS137138
                                "http://wustat.windows.com",  # DevSkim: ignore DS137138
                                "http://ntservicepack.microsoft.com",  # DevSkim: ignore DS137138
                                # "http://*.windowsupdate.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.windowsupdate.microsoft.com",  # Use absolute URL here
                                # "http://*.update.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.update.microsoft.com",  # Use absolute URL here
                                # "http://*.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.download.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.ws.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.ws.microsoft.com",  # Use absolute URL here
                                # "http://*.mp.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                            ],
                            "Service": "Microsoft Update Server",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                ],
                ignore_index=True,
            )
        elif self.cloud.lower() == "azuregermancloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://management.microsoftazure.de",
                                "https://management.core.cloudapi.de",
                            ],
                            "Service": "Azure Management Service",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "https://login.microsoftonline.de",
                            ],
                            "Service": "Authentication Service - Microsoft Entra ID",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ASE",
                            "URL": [
                                "http://windowsupdate.microsoft.com",  # DevSkim: ignore DS137138
                                "http://update.microsoft.com",  # DevSkim: ignore DS137138
                                "https://update.microsoft.com",
                                "http://download.microsoft.com",  # DevSkim: ignore DS137138
                                "https://download.microsoft.com",
                                "http://download.windowsupdate.com",  # DevSkim: ignore DS137138
                                "http://wustat.windows.com",  # DevSkim: ignore DS137138
                                "http://ntservicepack.microsoft.com",  # DevSkim: ignore DS137138
                                # "http://*.windowsupdate.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.windowsupdate.microsoft.com",  # Use absolute URL here
                                # "http://*.update.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.update.microsoft.com",  # Use absolute URL here
                                # "http://*.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.download.windowsupdate.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "http://*.ws.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                                # "https://*.ws.microsoft.com",  # Use absolute URL here
                                # "http://*.mp.microsoft.com",  # Use absolute URL here # DevSkim: ignore DS137138
                            ],
                            "Service": "Microsoft Update Server",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        }
                    ),
                ],
                ignore_index=True,
            )

    def load_nfm_specific_urls(self) -> None:
        """Load the NFM specific URLs."""
        # URLs for Geneva Monitoring and Telemetry service
        self.__data = pd.concat(
            [
                self.__data,
                pd.DataFrame(
                    {
                        "Cloud Environment": self.cloud,
                        "Layer": "NFM",
                        "URL": [
                            "https://" + self.region + "-prod.mecdevice.azure.com",
                        ],
                        "Service": "NFM Service",
                        "Status": "Not Tested",
                        "Reason": "NA",
                    }
                ),
            ],
            ignore_index=True,
        )

    def load_arc_specific_urls(self) -> None:
        """Load the ARC specific URLs."""
        if self.cloud.lower() == "azurecloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://"
                                + self.region
                                + ".dp.kubernetesconfiguration.azure.com",
                            ],
                            "Service": "Data Plane Endpoint",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://login.windows.net",
                                "https://" + self.region + ".login.microsoft.net",
                                "https://" + self.region + ".login.microsoft.com",
                            ],
                            "Service": "To Update ARM tokens",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://"
                                + self.region
                                + ".data.mcr.microsoft.com/v2/_catalog",
                            ],
                            "Service": "Container service for ARC Agent",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://gbl.his.arc.azure.com",
                                "https://"
                                + RegionAbbreviation[self.region]
                                + ".his.arc.azure.com",
                            ],
                            "Service": "System Assigned Managed Identity",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://k8connecthelm.azureedge.net",
                                "https://guestnotificationservice.azure.com",
                                "https://sts.windows.net",
                            ],
                            "Service": "For Cluster Connect and for Custom Location",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://graph.microsoft.com/",
                                "https://graph.windows.net",
                            ],
                            "Service": "Required when Azure RBAC is configured",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://" + self.region + ".obo.arc.azure.com:8084/",
                            ],
                            "Service": "Container service for ARC Agent",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://dl.k8s.io",
                            ],
                            "Service": "For Automatic Agent Upgrade",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                ],
                ignore_index=True,
            )
        elif self.cloud.lower() == "azureusgovernmentcloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://"
                                + self.region
                                + ".dp.kubernetesconfiguration.azure.us",
                            ],
                            "Service": "Data Plane Endpoint",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://" + self.region + ".login.microsoftonline.us",
                            ],
                            "Service": "To Update ARM tokens",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://"
                                + self.region
                                + ".data.mcr.microsoft.com/v2/_catalog",
                            ],
                            "Service": "Container service for ARC Agent",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://gbl.his.arc.azure.com",
                                "https://usgv.his.arc.azure.us",
                            ],
                            "Service": "System Assigned Managed Identity",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://k8connecthelm.azureedge.net",
                                "https://guestnotificationservice.azure.us",
                                "https://sts.windows.net",
                                "https://k8sconnectcsp.azureedge.net",
                            ],
                            "Service": "For Cluster Connect and for Custom Location",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://graph.microsoft.com/",
                                "https://graph.windows.net",
                            ],
                            "Service": "Required when Azure RBAC is configured",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://usgovvirginia.obo.arc.azure.us:8084/",
                            ],
                            "Service": "Cluster Connect",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://dl.k8s.io",
                            ],
                            "Service": "For Automatic Agent Upgrade",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                ],
                ignore_index=True,
            )

        elif self.cloud.lower() == "azurechinacloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://"
                                + self.region
                                + "dp.kubernetesconfiguration.azure.cn",
                            ],
                            "Service": "Data Plane Endpoint",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://" + self.region + ".login.chinacloudapi.cn",
                                "https://login.partner.microsoftonline.cn",
                            ],
                            "Service": "To Update ARM tokens",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://mcr.azk8s.cn/v2/_catalog",
                            ],
                            "Service": "Container service for ARC Agent",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://gbl.his.arc.azure.cn",
                                "https://sha.his.arc.azure.cn",
                            ],
                            "Service": "System Assigned Managed Identity",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://k8connecthelm.azureedge.net",
                                "https://guestnotificationservice.azure.cn",
                                "https://sts.chinacloudapi.cn",
                                "https://k8sconnectcsp.azureedge.net",
                            ],
                            "Service": "For Cluster Connect and for Custom Location",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://graph.chinacloudapi.cn",
                            ],
                            "Service": "Required when Azure RBAC is configured",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://" + self.region + ".obo.arc.azure.cn:8084/",
                            ],
                            "Service": "Cluster Connect",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://dl.k8s.io",
                            ],
                            "Service": "For Automatic Agent Upgrade",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://quay.azk8s.cn",
                                "https://registryk8s.azk8s.cn",
                                "https://k8sgcr.azk8s.cn",
                                "https://usgcr.azk8s.cn",
                                "https://dockerhub.azk8s.cn",
                            ],
                            "Service": "Container Registry for proxy server",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                ],
                ignore_index=True,
            )
        elif self.cloud.lower() == "azuregermancloud":
            self.__data = pd.concat(
                [
                    self.__data,
                    pd.DataFrame(
                        {
                            "Cloud Environment": self.cloud,
                            "Layer": "ARC",
                            "URL": [
                                "https://graph.cloudapi.de",
                            ],
                            "Service": "Required when Azure RBAC is configured",
                            "Status": "Not Tested",
                            "Reason": "NA",
                        },
                    ),
                ],
                ignore_index=True,
            )

    def load_custom_urls(self, urls: List[str]) -> None:
        """Load custom URLs provided by User."""

        # URLs for Microsoft Update Server
        self.__data = pd.concat(
            [
                self.__data,
                pd.DataFrame(
                    {
                        "Cloud Environment": self.cloud,
                        "Layer": "Custom URL",
                        "URL": urls,
                        "Service": "NA",
                        "Status": "Not Tested",
                        "Reason": "NA",
                    },
                ),
            ],
            ignore_index=True,
        )

    def retrieve(self) -> pd.DataFrame:
        """Retrieve the list of URLs."""
        return self.__data

    def get_urls_status(self) -> list[(str, str)]:
        """Retrieve the list of URLs with status."""

        status: list = []

        for layer in ["ASE", "ARC", "NFM", "PMEC"]:
            urls = self.__data[self.__data["Layer"] == layer]
            if urls.empty:
                status.append((layer, "Not Tested"))
            else:
                status_count = urls["Status"].value_counts()
                if "Fail" not in status_count:
                    status.append((layer, "Pass"))
                elif "Pass" not in status_count:
                    status.append((layer, "Fail"))
                else:
                    status.append((layer, "Partially Pass"))
        return status



# Copyright (c) Microsoft Corporation. All rights reserved.
# Highly Confidential Material
"""URL checker tool to validate the reqired URLs accessible for AP5GC orject which were added in Firewall rules added."""

from __future__ import print_function

import argparse
import logging
import os
import socket
import struct
import time

import urllib3

from pmec_url_checker.url_checker import UrlChecker

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
REPORT_FILE_DIR = os.path.expanduser("~")
REPORT_FILE_NAME = "url_checker_report.csv"
REPORT_FILE_PATH = os.path.join(REPORT_FILE_DIR, REPORT_FILE_NAME)


VERSION = "0.0.2"


def get_version() -> str:
    return (
        "URL Checker tool version: "
        + VERSION
        + "\n CopyRight (c) 2024 Microsoft Corporation. All rights reserved."
    )


def getNTPTime(hosts: list[str]):
    """Get the time from NTP Server."""
    port = 123
    buf = 1024
    msg = "\x1b" + 47 * "\0"

    for host in hosts:
        address = (host, port)

        # reference time (in seconds since 1900-01-01 00:00:00)
        TIME1970 = 2208988800  # 1970-01-01 00:00:00

        # connect to server
        client = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        client.settimeout(20)
        try:
            client.sendto(msg.encode("utf-8"), address)
            msg, address = client.recvfrom(buf)
        except socket.timeout:
            logging.debug(
                "FAIL: Timeout occured while connecting to NTP Server, check for other server endpoint"
            )
            continue
        except socket.error as e:
            logging.debug(
                "FAIL: Error occured while connecting to NTP Server: %s",
                e,
            )
            logging.debug("FAIL: check for other server endpoint.")
            continue
        else:
            logging.info("PASS: NTP Server is accessible.")
            client.close()
            t = struct.unpack("!12I", msg)[10]
            t -= TIME1970
            return time.ctime(t).replace("  ", " ")

    logging.info("FAIL: Unable to connect to NTP Server.")
    return None


def main() -> None:
    """Entry point for URL checker script."""

    parser = argparse.ArgumentParser(description="URL Checker tool for AP5GC")
    parser.add_argument(
        "-c",
        "--cloud",
        default="AzureCloud",
        metavar="<Azure Cloud type>",
        choices=[
            "AzureCloud",
            "AzureChinaCloud",
            "AzureUSGovernmentCloud",
            "AzureGermanCloud",
        ],
        help="Indicates the Azure environment. Required if the device is deployed to an environment other "
        "than the Azure public cloud (Azure Cloud). Supported values are [ AzureCloud | AzureChinaCloud | AzureUSGovernmentCloud | AzureGermanCloud ]",
    )
    parser.add_argument(
        "--custom-url",
        nargs="+",
        help="List ofs other URLs that you want to test HTTP access to.",
    )
    parser.add_argument(
        "-d",
        "--dns-server",
        required=True,
        nargs="+",
        metavar="<List of DNS Server>",
        help="IP addresses of the DNS servers (for example, your primary and secondary DNS servers) used for name resolution of URLs.",
    )
    parser.add_argument(
        "-o",
        "--output",
        type=str,
        metavar="<Output File Path>",
        default=REPORT_FILE_PATH,
        help="Dump the Result into file.",
    )
    parser.add_argument(
        "-r",
        "--region",
        type=str,
        default="eastus",
        metavar="<Region in small case without space>",
        help="Deployment Region ( default to eastus).",
    )
    parser.add_argument(
        "--skip-test",
        type=str,
        metavar="<Skip Test>",
        choices=[
            "ASE",
            "ARC",
            "NFM",
            "PMEC",
        ],
        nargs="+",
        help="skip checking URLs ( Supported values are [ASE, ARC, NFM, PMEC]).",
    )
    parser.add_argument(
        "-s",
        "--source-ip",
        type=str,
        metavar="<Source IP>",
        help="Source IP address to be used for querying the HTTP request.",
    )

    parser.add_argument(
        "-t",
        "--time-server",
        nargs="+",
        metavar="<List of NTP Server>",
        help="FQDN of one or more Network Time Protocol (NTP) servers used in ASE",
    )
    parser.add_argument(
        "-v",
        "--Verbose",
        action="count",
        default=0,
        help="Increase verbosity level",
    )

    parser.add_argument("--version", action="version", version=get_version())

    args = parser.parse_args()

    logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s")

    if args.Verbose >= 2:
        logging.getLogger().setLevel(logging.DEBUG)
    elif args.Verbose == 1:
        logging.getLogger().setLevel(logging.INFO)
    else:
        logging.getLogger().setLevel(logging.ERROR)

    is_ntp_server_accessible = []

    url_checker = UrlChecker(
        args.cloud,
        args.region if args.region else None,
        args.dns_server,
        args.source_ip if args.source_ip else None,
        args.custom_url if args.custom_url else None,
        args.skip_test if args.skip_test else None,
    )

    print("\nTesting the URL Endpoints.")

    # Now run the URL checker and check all the URLs presents in the URL DB class.
    url_checker.run()

    print("\n")

    # Now get the status of the URLs.
    status = url_checker.get_urls_status()
    for items in status:
        print("\tURL Endpoints for {}: {}".format(items[0], items[1]))

    # Now Dump the result into CSV file.
    url_checker.dump_result(args.output)

    print("\nTesting the NTP Server.")
    if args.time_server:
        logging.info("Checking the NTP Server")
        for time_server in args.time_server:
            resolved_ips = url_checker.get_time_server(time_server)
            if resolved_ips:
                received_time = getNTPTime(resolved_ips)
                if received_time is not None:
                    logging.debug("Time from NTP Server: %s", received_time)
                    is_ntp_server_accessible.append((time_server, True))
                else:
                    logging.error("Failed to get time from NTP Server.")
                    is_ntp_server_accessible.append((time_server, False))
            else:
                logging.error("Failed to resolve the NTP Server: %s", time_server)
                is_ntp_server_accessible.append((time_server, False))

    for items in is_ntp_server_accessible:
        print("\tNTP Server {}: {}".format(items[0], "PASS" if items[1] else "FAIL"))

    print("\nURL Checker completed successfully.")
    print(
        "To view the complete URL status in detail, View the Report at Location: {}".format(
            args.output
        )
    )


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Highly Confidential Material
"""URL checker tool to validate the reqired URLs accessible for AP5GC orject which were added in Firewall rules added."""

from setuptools import find_packages, setup

# read the contents of your README file
from pathlib import Path

VERSION = "0.0.2"
PACKAGE_NAME = "pmec-url-checker"

this_directory = Path(__file__).parent
long_description = (this_directory / "README_pypi.md").read_text()

DEPENDENCY = [
    "requests>=2.31.0",
    "dnspython>=2.6.1",
    "pandas>=2.2.1",
]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    packages=find_packages(),
    install_requires=DEPENDENCY,
    # url="<git repo url>", #project home page.
    license="Copyright (c) Microsoft Corporation. All rights reserved.",
    author="Sagar Bhatt",
    author_email="sagbhatt@microsoft.com",
    # download_url=<python Artifect url>
    description="URL checker tool for AP5GC.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.9",
    platforms="any",
    entry_points={
        "console_scripts": [
            "pmec-url-checker = pmec_url_checker.main:main",
        ],
    },
)
